/*
 * GuiDes source code:
 *
 * Copyright 2002 Carlos Andrs <carlos.andres@navegalia.com>
 * www - http://www.geocities.com/harbour_links/guides
 *
 * GuiDes is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * GuiDes is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 */

#include "hbclass.ch"
#include "GUIDES.CH"
#include "minigui.ch"

DECLARE WINDOW Main
DECLARE WINDOW OIWin
DECLARE WINDOW EditBar
DECLARE WINDOW PropInspector

CREATE CLASS gProps

   DATA gObject
   DATA nItem

   METHOD New
   METHOD Subst
   METHOD Refresh_Properties_Window
   METHOD EditProperty
   METHOD Properties

   METHOD PropName
   METHOD ChangeProp

   METHOD ApplyProperty
   METHOD EditColor
   METHOD SetFont
   METHOD EditTree 
ENDCLASS


METHOD New( newObject ) CLASS gProps

   ::Subst( newObject )
RETURN Self


METHOD Subst( newObject ) CLASS gProps
   ::gObject := newObject
   ::Refresh_Properties_Window()
RETURN NIL


METHOD Refresh_Properties_Window() CLASS gProps
   LOCAL nCount
   LOCAL ObjArray := ::Properties()
   LOCAL cPropValue := ""

   PropInspector.PropGrid.DeleteAllItems

   FOR nCount := 1 TO LEN( ObjArray )
     	cPropValue := CastToChar( ObjArray[nCount,2] )
     	PropInspector.PropGrid.AddItem( { ObjArray[nCount,1], cPropValue } )
   NEXT
RETURN NIL


METHOD EditProperty() CLASS gProps
   LOCAL wIndex := PropInspector.PropGrid.Value
   LOCAL ObjArray := ::Properties()
   LOCAL aPropLine, cPropName

   IF (wIndex == 0) .OR. (wIndex > LEN( ObjArray ))
      PlayHand()
   ELSE
      ::nItem   := wIndex
      aPropLine := ObjArray[ wIndex ]
      cPropName := ALLTRIM(UPPER(aPropLine[1]))
      xValue    := aPropLine[2]

      IF ( cPropName == "NAME" ) ;
       .OR. ( cPropName == "OPAREN" ) ;
       .OR. ( cPropName == "TYPE" )
         MsgInfo( cPropName + " property cannot be modified.", "GuiDes" )

      ELSEIF ( cPropName == "FONT" )
         ::SetFont()

      ELSEIF ( "COLOR" $ cPropName )
         ::EditColor()

      ELSEIF ( "ELEMENTS" $ cPropName )
         ::EditTree()

      ELSEIF ( VALTYPE( xValue ) == "L" )
         ::ApplyProperty( TRUE )

      ELSE
	   PropEditWin( xValue, cPropName, ::gObject:Id , ::gObject:oParen:Id )		

      ENDIF
   ENDIF
RETURN NIL


METHOD Properties() CLASS gProps
    IF ::gObject == NIL
        RETURN {}
    ENDIF
RETURN ( __objGetValueList( ::gObject ) )


METHOD PropName() CLASS gProps
RETURN ::Properties()[ ::nItem ][ 1 ]


METHOD ChangeProp( xRef , xValue )     // xRef can be either array index  
   LOCAL cPropName              // (number) or property name (character)
   LOCAL aProps                        

   IF VALTYPE( xRef ) == "N"
      ::nItem := xRef
      cPropName := ::PropName()
   ELSE
      aProps := ::Properties()
      cPropName := ALLTRIM(UPPER(xRef))
      ::nItem := ASCAN( aProps, { |aRow| aRow[1] == cPropName } )
   ENDIF

   __objSendMsg( ::gObject, "_" + cPropName, xValue )
   PropInspector.PropGrid.Item( ::nItem ) :=  { cPropName, CastToChar( xValue ) }
RETURN Self


METHOD ApplyProperty( lApply ) CLASS gProps
   LOCAL OldValue
   LOCAL xValue

   IF lApply
      OldValue := ::Properties()[ ::nItem ][ 2 ]

      IF VALTYPE( OldValue ) == "C"
         xValue := EditBar.cEditBox.Value
         EditBar.Release

      ELSEIF VALTYPE( OldValue ) == "N"
         xValue := EditBar.NSpinner.Value
         EditBar.Release

      ELSEIF VALTYPE( OldValue ) == "L"
         xValue := !( OldValue )
         MsgInfo( ::PropName +  " property will be set to" + CR_LF + CR_LF + IIF( xValue, "TRUE", "FALSE" ) , "GuiDes" )
      ELSE
         MsgBox("Unknown value type!", "GuiDes")
      ENDIF


      ::ChangeProp( ::nItem, xValue )
      ::gObject:Update()
      ::gObject:oParen:SetChanged( TRUE )     
   ELSE
      IF !( VALTYPE( OldValue ) $ "L" )
		EditBar.Release
	ENDIF
   ENDIF

RETURN NIL


METHOD EditColor() CLASS gProps
   LOCAL oColor  := GetColor()

   ::ChangeProp( ::nItem, oColor )
   ::gObject:Update()      
   ::gObject:oParen:SetChanged( TRUE )     
RETURN NIL


METHOD SetFont() CLASS gProps
   LOCAL aFont := ChangeFont( ::gObject:Font )

   ::ChangeProp( ::nItem, aFont )
   ::gObject:Update()      
   ::gObject:oParen:SetChanged( TRUE )     
RETURN NIL


METHOD EditTree() CLASS gProps
   LOCAL aElements := ::Properties()[ ::nItem ][ 2 ]
   LOCAL oMenu := gMenu():New( G_TREE )

   oMenu:Elements := aElements 
   oMenu:Edit()

   IF oMenu:ExitStatus == G_EXIT_DESTROY
       oMenu:Elements := {}  
   ENDIF

   ::ChangeProp( ::nItem, oMenu:Elements )
   ::gObject:Update()      
   ::gObject:oParen:SetChanged( TRUE )     
RETURN NIL
